%basicTrackingScript

% ---------- Input Directories of relevant folders/files ------------ %
% > "imgFiles": Directory of folder containing images extracted from video
%             in a standard image file type e.g. .tiff or .jpg
% > Example input: 'C:\Documents\Experiment_video_image_stack_folder';
imgFiles = '';

% > "frameTimesTable": Directory of excel file (.xlsx) containg times  
%                      corresponding to relevant video frames. 
% > File layout > Column A: video name or file name. Not important what
%                              the name is.
%               > Column B: Frame time relative to video start.
%               > Column C: Frame number according to order (1 = 1st frame).
%               > Row 1: Reserved for column labels/names.
% > Example input: 'C:\Documents\Experiment_table_of_frame_times_excel_file.xlsx';
frameTimesTable = '';
% ----------------------------------------------------------- %

imgFolderTest = imgFiles;
newIm3 = getGrayscale(imgFiles);
newIm3I = 255-newIm3; % <- to invert the high (light/white) and low (dark/black) image values

% ------------ electrodeMasker ------------- %
% > electrodeMasker(image,paramsMsk)

% > image: grayscale image with intensity values between 0-255.

% > paramsMsk: structure containing relevant optional input for controling
%              the mask generating process.

% > paramsMsk.blurFactor: Used as the params.deviation argument for imgFilterF. 
paramsMsk.run1.blurFactor = 6;
paramsMsk.run2.blurFactor = 7;

% > paramsMsk.threshold: Used as the cuttOff argument for masker.
paramsMsk.run1.threshold = 170;
paramsMsk.run2.threshold = 30;

% > paramsMsk.inverse: For a grescale image with intensity values between 0-255; true will
%                      result in an inverse in intensity values. low-->high/high-->low.
paramsMsk.inverse = false;

% > paramsMsk.logical: If true, the output is a logic/bool 01 matrix, if false then the output
%                      output is a numerical 01 matrix.
paramsMsk.logical = true;

% > paramsMsk.inputMod: If true, 'params.run2.threshold' input is internaly divided by a 
%                       constnt (255). If false no modification is carried out. 
paramsMsk.inputMod = true;

electrodeMskA = electrodeMasker(newIm3,paramsMsk); % <- sub mask A

paramsMsk.run1.blurFactor = 6;
paramsMsk.run2.blurFactor = 15;
paramsMsk.run1.threshold = 170;
paramsMsk.run2.threshold = 20;
paramsMsk.inverse = false;
paramsMsk.logical = true;
paramsMsk.inputMod = true;

electrodeMskB = electrodeMasker(newIm3I,paramsMsk);  % <- sub mask B

% > Note: Here two sub masks are created and merged together to cover different 
%         areas of the frame (e.g. to block the electrode and capillary wall). 
%         It is not always nessasary to combine 2 masks to make the final mask,
%         this is just given as an example to show how one might use the mask 
%         creation functions to generate more complex masks. 

electrodeMsk = electrodeMskA|electrodeMskB; % <- Combining blocked areas of two sub masks.

imagesc(electrodeMsk); % <- to visualise the resultant mask
% imagesc(newIm3); % <- uncomment to instead veiw the image used to make
%                  % the mask (to check if electrode/obstacles are coverd or not)

%%

% ------------ parameterChecker ------------- %
% > parameterChecker(targetParameters, imgFiles, electrodeMsk)

% > imgFiles: Either a greyscale image with intensity ranges from 0 - 255 or 
%             a folder directory containint the image. Should be the same
%             image which was used to make the mask.
 
% > electrodeMsk: (optional) - logical matrix with 1 representing blocked pixels and 0 for accesible pixels.
 
% > "parameterChecker": Function for checking sets of parameters which will 
%                       be used in order to locate bacteria in images 

% > "targetParameters": cell array of input parameters used by "parameterChecker".
% > Layout: cell array with 5-7 columns and a variable number of rows.
%           each row represents a seperate parameter set used in the
%           findObjectX function.
%          --- REQUIRED ---
%          -> Column 1 = filterA deviation (lowerBound length) [e.g. 2].
%          -> Column 2 = filterB deviation (upperBound length) [e.g. 3].
%          -> Column 3 = intensity threshold [e.g. 4.5].
%          -> Column 4 = exclusion length (clossest distance in pixels 
%                       2 points found using the same parameters can be) [e.g. 20].
%          -> Column 5 = invert (i.e. 255 - image) [e.g. false].
%          --- OPTIONAL ---
%          ~> Column 6 = plot color [e.g. 'r' (red color)].
%          ~> Column 7 = plot symbol [e.g. 'o' (circle marker)].

% > Example 1: Using only one set of parameters.
% targetParameters = {3,4,3.5,20,true,'r','o'};

% > Example 2: Using two sets of parameters.
% targetParameters = {2,3,4.5,20,false,'r','o';
%                     2,3,3.5,20,true,'b','^'};

% > Example 3: Using three sets of parameters.
targetParameters = {2,3,4,20,false,'r','o';
                    3.5,4.5,2.5,20,false,'g','s';
                    2,3,4.0,20,true,'b','^'};
                
% > Advice: little advantage in using more than 3 sets of parameters
%           (processing time will increase significantly per set).

parameterChecker(targetParameters, imgFiles, electrodeMsk)



%%
% ====================== getPos =================== %
% > getPos(targetParameters, imgFolderTest, electrodeMsk, frameTimesTable, params)

% > getPos > For locating a table of bacterial positions and relevent info
%            from a stack of frames.
%          > Returns a compatible output for putting into the tracker functions.
%  
% > Note: "getPos" is mainly just a wraper for "getLocations"

% ==== Input ==== % 
% --- Required --- %
%  
% > targetParameters > cell array with 5-7 columns and variable rows.
%                    > each row represents a seperate parameter set used in the
%                       findObjectX function.
%         [required] > Column 1 = filterA deviation (lowerBound length) <2>.
%         [required] > Column 2 = filterB deviation (upperBound length) <3>.
%         [required] > Column 3 = intensity threshold <4.5>.
%         [required] > Column 4 = exclusion length (clossest distance in pixels 
%                       2 points found using the same parameters can be) <20>.
%         [required] > Column 5 = invert (i.e. 255 - image) <false>.
% 
% > imageFolder > address of folder containing image files.
% 
% --- optional --- %
% 
% NOTE - important to set "params.interExclude.exclusionLength".
% NOTE - Under most circumstances a mask shold be provided for "maskX".
% NOTE - Under most circumstances an appropriate excell file containing 
%        frame time information shold be provided for "frameTimeFile".
% 
% > frameTimeFile > excel file with > col 1 = name of image file. 
%                                   > col 2 = time signiture from start in ms.
%                                   > col 3 = frame number/idx starting at 1.
%                                   > 1st row contains column names/titles.
% 
% > maskX: bool/01 matrix with 1 representing blocked pixels and 0 for
%          accesible pixels.
% 
% > params.interExclude.exclusionLength > minimum number of pixels between points from different
%                                          sets allowed. if 2 points close to each other, the 
%                                          brightest will be selected.
%                                       > Default value = roughly 20 pixels.
% 
% > params.interExclude.useIntensity > Default varies depending on if the are > 2 columns
%                                    > If set to false, will not use intensity even if
%                                       available.
% 
% > params.interExclude.prioritiseA: - Default value = true (i.e. prioritise A over B). if false 
%                                    - setB is prioritised over set A.
% 
% > params.interExclude.accurate > Default value = false.
%                                > false -> use faster, less acurate measuring method. 
%                                > true -> use slower, more acurate measuring method.
% 
% > params.startIndex > instead of processing from the start, will process
%                     > from the given index untill reaching end or input endIndex.
% 
% > params.endIndex > instead of processing to the last file, will process
%                   > up untill the given end index.
% 
% > params.realFrameStart > default is either 1 or "params.startIndex".
%                         > The defaults refer to the file index which is not 
%                         > necessarily the same as the frame index.
%                         > set this if the file index and the frame index 
%                           start differ. 
% 
% > params.quiet > Default value = true -> Supreses the diplaying of processing time. 
%                > false -> Processing time is displayed.
% 
% > params.output > string to indicate the output type.
%                 > 'raw' > output is the output of getLocations sub function.
%                 > 'standard' > output is similar to posFinder.
%                              > Default output.
%                 > 'new' > same as standard but with an extra 6th column
%                            containing normalised intensities.
% 
% ==== Output ==== %
% 
% positionList > 2 column cell array > col 1 = cells each containing a frames
%                                      [X,Y,Intensity] data for all located 
%                                      positions.
%                                    > col 2 = the number of possitions
%                                      located in each frame.

[positionList] = getPos(targetParameters, imgFolderTest, electrodeMsk, frameTimesTable);
%%
% ---------- Trace formation ----------- %

% Note - Max displacement = maximum distance (in pixels) allowed between
%        consecutive positions within a trace.

paramsTa.paraPre = struct; % B > base struct for pre treatment i.e. the low extremities
paramsTa.paraNorm = struct; % A > base struct for main track finding iterations
paramsTa.paraEnd = struct; % C > base struct for high end extremities treatment

paramsTa.preDisp = 7; % [3,5]; %[2,3]; %4; % > Pre treatment max displacement. 
paramsTa.preThreshold = 0.03; % 0.025; % > Pre treatment cutoff average velocity (px/ms) > use low values in order to only remove stationary artefacts  
paramsTa.preTreat = 1; % 2; % > The number of pre treatment iterations (0, 1 or 2)
paramsTa.thresholdConstant = 1; % 1.5; % > factor to multiply the automaticaly calculated velocity threshold cut off point.
paramsTa.firstDisp = 30; % 1.5; % > Initial max displacement allowed for main itterations (after any pretretment). 
paramsTa.paraPre.mem = 100; % > B > number off frames between consecutive positions allowed in a trace <- for pre treatment i.e. the low extremities
paramsTa.paraPre.good = 15; % > B > minimum positions required for a legitamate trace <- for pre treatment i.e. the low extremities
paramsTa.paraNorm.mem = 4; % > A > number off frames between consecutive positions allowed in a trace <- for main track finding iterations
paramsTa.paraNorm.good = 5; % > A > minimum positions required for a legitamate trace <- for main track finding iterations
paramsTa.paraEnd.mem = 3; % > C > number off frames between consecutive positions allowed in a trace <- for large maxDisp track finding iterations i.e. the high end extremities
paramsTa.paraEnd.good = 4; % > C > minimum positions required for a legitamate trace <- for large maxDisp track finding iterations i.e. the high end extremities
maxDisplacementTa = 45; % > upper limit for max displacement distance (pixels) (aprox. highest posible distance between consecutive positions in a trace).

[traces1, raw1, resultsBox1] = trackPosModV6(positionList,maxDisplacementTa,paramsTa); % <40> % A=50 % B=30 % C=30?
%%
% ------------ Plot all positions ------------ %
colormap('gray'), imagesc(newIm3);
hold on
plot(positionList(:,1),positionList(:,2),'rx');
%%
% ------------ Plot all calculated traces ------------ %
colormap('gray'), imagesc(newIm3);
hold on
for k=1:numel(traces1(:,1));   
    plot(traces1{k}(:,1),traces1{k}(:,2),'r')
    hold on
end
%%
% ------------ Remove stationary traces (optional) ------------ %
[posC,traces0C,purgedTraces,purgedRaw] = ratePurgeV3(traces1,positionList,0.02); % <0.02>

%%
% ------------ Plot only traces above a certain average velocity ------------ %
colormap('gray'), imagesc(newIm3);
hold on
for k=1:numel(purgedTraces(:,1));    
    plot(purgedTraces{k}(:,1),purgedTraces{k}(:,2),'b')
    hold on
end
