% excludeMasked(XYpositionList,mask,options)
%
%
% [newPositionTable] = excludeMasked(XYpositionList,mask,options)
%
% EXPLANATION:
% > XYpositionList must have at least 2 columns. returned position list
% newPositionTable will include all of the original columns.
%
% <><><> Required Functions <><><> %
% > N/A
%
% ================ INPUTS ================ %
%
% ---- Required ---- %
% > XYpositionList: XY coordinate location list where column 1 = X
%           coordinate and column 2 = Y coordinate.
%           > (X,Y) = (col,row)
% > mask: logic/01 mask. position corrosponding to pixels set to one are 
%           from the returned position table.
%
% - note - Important values to set: 
% --> N/A
%
% ---- Optional 1 ---- %
% > N/A
%
% ---- Optional 2 ---- %
% > N/A
% ======= OUTPUT ======= %
% > newPositionTable: same as input list minus the excluded rows.
%


function [newPositionTable] = excludeMasked(XYpositionList,mask,options)

% ~~~~~ Inputs ~~~~~ %

% Defaults



if nargin > 1
    maskE = mask;
else
    disp('Error: call to "excludeMasked" is missing required arguments');
    return;
end

[inRow, inCol] = size(XYpositionList);

if inCol < 2
    disp('Error: parameter "XYpositionList" used call "excludeMasked" is missing required columns');
    return;
end

tRow = inRow;
tCol = inCol + 2;
tempList = zeros(tRow,tCol);
tempList(:,1:inCol) = XYpositionList(:,:);
tEa = tCol-1; % <-- second to last column.
tEb = tCol; % <-- last column.



    
% ------- Exclusion of points from mask area -------- % 
if length(maskE(1,:)) > 1 % checking if maskE contains a matix as aposed to a single boolean 
    maskI = ~maskE; % invert 0's and 1's.

    [maskRows,maskCols] = size(maskI);
%     positionTempB = zeros(length(positionTemp(:,1)),4);
%     positionTempB(:,1:2) = positionTemp(:,1:2);
    tempList(:,tEa) = ((XYpositionList(:,1) - 1)*maskRows) + XYpositionList(:,2); % create 1D index from 2D index, using the mask size.
    % becaus pkfnd gives as x(col) y(row), whilst matlab works with row,col
    tempList(:,tEb) = maskI(tempList(:,tEa)); 
    newPositionTable = tempList(tempList(:,tEb) == 1, 1:inCol);
end


end