% findObjectsX()
%
% [positionTable] = findObjectsX(image, paramList, options)
%
% <><><> Required Functions <><><> %
% > imgFilterF.m
% > pkfnd.m
%
% ======== INPUTS ======== %
% ---- Required ---- %
% > image: intensity matrix.
%
% - note - Important values to set: 
% --> paramList.A.deviation
% --> paramList.B.deviation
% --> paramList.threshold
% --> paramList.exclusionLength
%
% ---- Optional 1 ---- %
% > paramList.invert (false): if true flips the greyscale intensities
% > paramList.A.________: filter 1 = low range filter (~remove noise)
% > paramList.B.________: filter 2 = high range filter (~remove details)
% > paramList.__.deviation = related to the feature size. 
%   filter 1 (paramList.A.deviation = 2) < filter 2 (paramList.B.deviation = 3)
% > paramList.__.kernelWidth (100).
%   Should not be changed usualy.
% > paramList.threshold (4.5):
% --> Refer to "th" explanation in pkfnd.m.
% > paramList.exclusionLength (20):
% --> Refer to "sz" explanation in pkfnd.m.
%
% ---- Optional 2 ---- %
% > options.mask (false) :- logic mask with ones and zeros. ones = bloked
%   area.
% > options.quiet (false):- prevents display of results.
% > options.kernalType ('gaussian'):- type of kernal to apply. 
% > options.processType ('fourier'):- type of algorithim used to apply kernel.
% - Unless know what doing, leave "options.kernalType" and "options.processType"
%   at their default values!
% ======= OUTPUT ======= %
% positionTable: col 1 = x, col 2 = y, col3 = intensity
%


function [positionTable] = findObjectsX(image, paramList, options)

% ~~~~~ Inputs ~~~~~ %

% Defaults
quiet = false;                  % Changeable
maskE = false;                  % Changeable
kernalType = 'gaussian';        % Changeable but best kept at default
processType = 'fourier';        % Changeable but best kept at default
pStruct = struct;
pStruct.invert = false;
pStruct.A = struct;
pStruct.B = struct;
pStruct.A.deviation = 2;        % Changeable
pStruct.B.deviation = 3;        % Changeable
pStruct.A.pad = 'reflection';   % Fixed
pStruct.B.pad = 'reflection';   % Fixed
pStruct.A.kernelWidth = 100;    % Changeable but best kept at default 100
pStruct.B.kernelWidth = 100;    % Changeable but best kept at default 100
pStruct.threshold = 4.5;        % Changeable % e.g 2.8 or 3.2 
pStruct.exclusionLength = 20;   % Changeable 

% --- Optional 1 (paramList) --- %
if nargin > 1
   if isfield(paramList,'A')
       if isstruct(paramList.A)
           if isfield(paramList.A,'deviation')
                pStruct.A.deviation = paramList.A.deviation;
           end
           if isfield(paramList.A,'kernelWidth')
                pStruct.A.kernelWidth = paramList.A.kernelWidth;
           end
       end
   end
   if isfield(paramList,'B')
       if isstruct(paramList.B)
           if isfield(paramList.B,'deviation')
                pStruct.B.deviation = paramList.B.deviation;
           end
           if isfield(paramList.B,'kernelWidth')
                pStruct.B.kernelWidth = paramList.B.kernelWidth;
           end
       end
   end
   if isfield(paramList,'threshold')
       pStruct.threshold = paramList.threshold;
   end
   if isfield(paramList,'exclusionLength')
       pStruct.exclusionLength = paramList.exclusionLength;
   end
end

% --- Optional 2 (options) --- %
if nargin > 2
   if isfield(options,'quiet')
       quiet = options.quiet;
   end
   if isfield(options,'kernalType')
       kernalType = options.kernalType;
   end
   if isfield(options,'processType')
       processType = options.processType;
   end
   if isfield(options,'mask')
       maskE = options.mask;
   end
end

% ~~~~~ Inputs END ~~~~~ %

% -A-
paramA.deviation = pStruct.A.deviation;
paramA.kernelWidth = pStruct.A.kernelWidth;
paramA.pad = pStruct.A.pad;
paramA.quiet = true;
% -B-
paramB.deviation = pStruct.B.deviation;
paramB.kernelWidth = pStruct.B.kernelWidth;
paramB.pad = pStruct.B.pad;
paramB.quiet = true;

if (pStruct.invert)
    dataImage = 255-image;
else
    dataImage = image;
end
    
deltaImageA = imgFilterF(dataImage, kernalType, processType, paramA);
deltaImageB = imgFilterF(deltaImageA, kernalType, processType, paramB);
filteredImg = deltaImageA - deltaImageB;
% ----- Aoutomatic calculation of threshold parameters ----- % 
% requires implentation
maxVal = max(filteredImg(:));
minVal = min(filteredImg(:));
rangeVal = maxVal - minVal;
% ------ section END ------ %

positionTemp = pkfnd2(filteredImg, pStruct.threshold, pStruct.exclusionLength); 

[tempR,tempC] = size(positionTemp);
tempBC = tempC + 2;
% ------- Exclusion of points from mask area -------- % 
if length(maskE(1,:)) > 1 % checking if maskE contains a matix as aposed to a single boolean 
    maskI = ~maskE; % invert 0's and 1's.

    [maskRows,maskCols] = size(maskI);
    positionTempB = zeros(length(positionTemp(:,1)),tempBC);
    positionTempB(:,1:tempC) = positionTemp(:,1:tempC);
    positionTempB(:,(tempBC-1)) = ((positionTempB(:,1) - 1)*maskRows) + positionTempB(:,2); % create 1D index from 2D index, using the mask size.
    % becaus pkfnd gives as x(col) y(row), whilst matlab works with row,col
    positionTempB(:,tempBC) = maskI(positionTempB(:,(tempBC-1))); 
    positionTable = positionTempB(positionTempB(:,tempBC) == 1, 1:tempC);
else % ------- No mask exclusion -------- % 
    positionTable = positionTemp;
end

if ~quiet
    
    
end


end


% --------- backup prior to modification ------------ %
% 
% positionTemp = pkfnd(filteredImg, pStruct.threshold, pStruct.exclusionLength); 
% 
% 
% % ------- Exclusion of points from mask area -------- % 
% if length(maskE(1,:)) > 1 % checking if maskE contains a matix as aposed to a single boolean 
%     maskI = ~maskE; % invert 0's and 1's.
% 
%     [maskRows,maskCols] = size(maskI);
%     positionTempB = zeros(length(positionTemp(:,1)),4);
%     positionTempB(:,1:2) = positionTemp(:,1:2);
%     positionTempB(:,3) = ((positionTempB(:,1) - 1)*maskRows) + positionTempB(:,2); % create 1D index from 2D index, using the mask size.
%     % becaus pkfnd gives as x(col) y(row), whilst matlab works with row,col
%     positionTempB(:,4) = maskI(positionTempB(:,3)); 
%     positionTable = positionTempB(positionTempB(:,4) == 1, 1:2);
% else % ------- No mask exclusion -------- % 
%     positionTable = positionTemp;
% end
% 
% if ~quiet
%     
%     
% end