% imgFilterF
%%% REQUIRED ARGUMENTS
% imageA % in the form of a 2D intensity matrix
% filter % 'gaussian','custom','preMade','box'
% method % 'fourier', 'simple'
%%% DEPENDING ON TYPE OF FILTER
% params = {};
% params.kernel % A.K.A kernelNum % positive integer between 1-4 to select desired kernel % required if using type = 'preMade'.
% params.pad % 'zero','reflection','reflection1','reflection2','reflection3','mirrorX4' *note 'reflection' defaults to 'reflection1/2' depending on other parameters. if blank, zero padding is used.
% params.deviation % A.K.A stD % can be any positive number. Higher number --> broader gaussian % required if using type = 'gaussian'.
% params.kernelWidth % A.K.A gussiankernelWidth % any odd integer or 'auto' or 'same' % semi-required if using type = 'gaussian', if not given will default to 'auto'
% params.modifier % 'inverse' % modifies the filter in "fourrier space"
% params.kernelStatus % 'fft','pure' % 'pure' = no normilisation or fft carried out on kernel before the kernel is applied to the image % 'fft' = extra fft used on the kernel where usefull
% params.boxRadius
% params.quiet % true or false. Defaults to false.

function [newImg,kernel,FTpadKern,padResult] = imgFilterF(imageA,filter,method,params)

type = filter;

blockGraph = false;
invertKern = false; % NEW
specialPad = false; % NEW
reflectPad = false;
untainted = false;
extraFFT = false;
customX = false;
customY = false;
keepSize = false;
zeroPad = false;

if nargin == 4 && exist('params','var') == 1
    
    if isfield(params,'modifier') == 1 % NEW
        if strcmp(params.modifier, 'inverse') % NEW
            invertKern = true;
        end
    end
    if isfield(params, 'quiet') == 1
        if params.quiet == true
            blockGraph = true;
        end
    end
    if isfield(params, 'pad') == 1
        if strcmp(params.pad,'reflection')
            reflectPad = true;
            if invertKern == true
                reflectType = 2;
            else
                reflectType = 1;
            end
        elseif strcmp(params.pad,'reflection1')
            reflectPad = true;
            reflectType = 1;
        elseif strcmp(params.pad,'reflection2')
            reflectPad = true;
            reflectType = 2;
        elseif strcmp(params.pad,'reflection3')
            reflectPad = true;
            reflectType = 3;
        elseif strcmp(params.pad,'mirrorX4') % NEW
            specialPad = true;
            specialType = 1;
        elseif strcmp(params.pad,'zero') % NEW
            zeroPad = true;
            specialType = 2;
        end
    end
    
    if isfield(params, 'kernel') == 1
        kernelNum = params.kernel;
    else
        kernelNum = 1;
        if strcmp(type,'preMade') == 1
            disp('if you want to select a preMade kernel you must add an extra field to params; params.kernel, with a number between 1-4')
        end
    end
    if isfield(params, 'deviation') == 1
        stD = params.deviation;
    else
        stD = 1;
    end
    if isfield(params, 'kernelWidth') == 1
        generalkernelWidth = params.kernelWidth;
    end
    if isfield(params, 'kernelStatus') == 1
        if strcmp(params.kernelStatus,'pure') == 1
            untainted = true;
        elseif strcmp(params.kernelStatus,'fft') == 1
            extraFFT = true;
        end
    end
    if isfield(params, 'boxCentreX') == 1
        bxMidX = params.boxCentreX;
        customX = true;
    end
    if isfield(params, 'boxCentreY') == 1
        bxMidY = params.boxCentreY;
        customY = true;
    end
end

oldImg = imageA;
imgWidth = size(oldImg,2); % number of columns
imgHight = size(oldImg,1); % number of rows

if strcmp(type,'box') == 1
    
    if exist('params','var') == 1
        if isfield(params, 'boxRadius') == 1
            boxR = params.boxRadius;
        else
            boxR = 9;
            disp('require "params.boxRadius" to set the radius of the boxcar filter. Radius defaulted to 9');
        end
    else
        boxR = 9;
        disp('require "params.boxRadius" to set the radius of the boxcar filter. Radius defaulted to 9');
    end
    
    if untainted == true
        keepSize = true;
        GkernW0 = imgWidth;
        GkernH0 = imgHight;
    else
        if exist('generalkernelWidth','var')
            if ischar(generalkernelWidth)
                if strcmp(generalkernelWidth,'auto')
                    GkernW0 = round(boxR*4);
                    GkernH0 = GkernW0;
                elseif strcmp(generalkernelWidth,'same')
                    GkernW0 = imgWidth;
                    GkernH0 = imgHight;
                else
                    disp('The params.kernelWidth argument you gave is not acceptable');
                end
            elseif isnumeric(generalkernelWidth)
                if length(generalkernelWidth) > 1
                    GkernH0 = round(generalkernelWidth(1));
                    GkernW0 = round(generalkernelWidth(2));
                else
                    GkernW0 = round(generalkernelWidth);
                    GkernH0 = GkernW0;
                end
            end
        else
            GkernW0 = round(boxR*4);
            GkernH0 = GkernW0;
        end
    end
    if customX == false
        bxMidX = ceil(GkernW0/2);
    end
    if customY == false
        bxMidY = ceil(GkernH0/2);
    end
    tempC = circleF(boxR,GkernW0,GkernH0,bxMidX,bxMidY);
    
    bxKern = tempC;
    
    if untainted == false
        %%% NORMALISATION of kernel %%%
        bksum = sum(bxKern(:));
        for il = 1:length(bxKern(:))
            bxKern(il) = bxKern(il)/bksum;
        end
        kernel = bxKern;
        %%% END %%%
    elseif untainted == true
        if invertKern == true 
            kernel = 1-bxKern;
        else
            kernel = bxKern;
        end
    end
end

if strcmp(type,'gaussian') == 1
    gussKernAdjust = true;
    if exist('generalkernelWidth','var')
        if ischar(generalkernelWidth)
            if strcmp(generalkernelWidth,'auto')
                GkernW0 = round((stD*3 + 2)); % require better method for selecting kernel size.
                GkernH0 = GkernW0;
            elseif strcmp(generalkernelWidth,'same')
                GkernW0 = imgWidth;
                GkernH0 = imgHight;
                gussKernAdjust = false;
            else
                disp('The deviation argument you gave is not acceptable');
            end
        elseif isnumeric(generalkernelWidth)
            if length(generalkernelWidth) > 1
                GkernH0 = round(generalkernelWidth(1));
                GkernW0 = round(generalkernelWidth(2));
            else
                GkernW0 = round(generalkernelWidth);
                GkernH0 = GkernW0;
            end
        end
    else
        GkernW0 = round((stD*3 + 2)); % require better method for selecting kernel size.
        GkernH0 = GkernW0;
    end
    if gussKernAdjust == true
        if GkernH0 < 3
            GkernH0 = 3;
        end
        if rem(GkernH0,2) > 0
            GkernH = GkernH0;
        else
            GkernH = GkernH0 + 1;
        end
        if GkernW0 < 3
            GkernW0 = 3;
        end
        if rem(GkernW0,2) > 0
            GkernW = GkernW0;
        else
            GkernW = GkernW0 + 1;
        end
    else
        GkernH = GkernH0;
        GkernW = GkernW0;
    end

    Gkern = zeros(GkernH,GkernW);
    GknCol = size(Gkern,2);
    GknRow = size(Gkern,1);

    mvCol = @(a) (a-1)-floor(GknCol/2);
    mvRow = @(a) floor(GknRow/2)-(a-1);

    for irow = 1:GknRow
        for icol = 1:GknCol
            Gkern(irow,icol) = myGausian(mvCol(icol),mvRow(irow),stD);
        end
    end
%     if invertKern == true && untainted == false
    if untainted == false
        %%% NORMALISATION of guassian kernel %%%
        Gksum = sum(Gkern(:));
        for il = 1:length(Gkern(:))
            Gkern(il) = Gkern(il)/Gksum;
        end
        kernel = Gkern;
        %%% END %%%
    elseif untainted == true
        if invertKern == true
            kernel = 0 - Gkern;
        else
            kernel = Gkern;
        end
    end
end
        

if strcmp(type,'preMade')
    if exist('kernelNum','var')
        if kernelNum == 1
            kernel = [0,0,-1,0,0;...
                     0,0,-1,0,0;...
                     0,0,4,0,0;...
                     0,0,-1,0,0;...
                     0,0,-1,0,0];
        elseif kernelNum == 2
            kernel = [0,0,-1,0,0;...
                     0,0,-1,0,0;...
                     -1,-1,8,-1,-1;...
                     0,0,-1,0,0;...
                     0,0,-1,0,0];
        elseif kernelNum == 3
            kernel = [0,0,-1,0,0;...
                     0,0,-1,0,0;...
                     0,0,2,0,0;...
                     0,0,0,0,0;...
                     0,0,0,0,0];
        elseif kernelNum == 4
            kernel = [0,0,0,0,0;...
                     0,0.0625,0.0625,0.0625,0;...
                     0,0.0625,0.5,0.0625,0;...
                     0,0.0625,0.0625,0.0625,0;...
                     0,0,0,0,0];
        elseif kernelNum == 5
            kernel = [-0.03,-0.03,-0.03,-0.03,-0.03;...
                     -0.03,0.0625,0.0625,0.0625,-0.03;...
                     -0.4,0.0625,1,0.0625,-0.4;...
                     -0.03,0.0625,0.0625,0.0625,-0.03;...
                     -0.03,-0.03,-0.03,-0.03,-0.03];
        elseif kernelNum == 6
            kernel = [0,0,1,0,0;...
                     0,0,1,0,0;...
                     0,0,-4,0,0;...
                     0,0,1,0,0;...
                     0,0,1,0,0];
        elseif kernelNum == 7
            kernel = [-1,0,1,0,-1;...
                     0,2,-2,2,0;...
                      1,-2,1,-2,1;...
                     0,2,-2,2,0;...
                     -1,0,1,0,-1];
        end
        if untainted == true && invertKern == true
            kernel = 0 - kernel;
        end
    end
end

%%% impulse response of kernel %%%



%%% END %%%

kernW = size(kernel,2);
kernH = size(kernel,1);

if strcmp(method,'fourier')
    
    if keepSize == true || untainted == true
        padImg = oldImg;
        padIW = imgWidth; % NEW
        padIH = imgHight; % NEW
        remvPad = @(a) a(:,:);
    else
        if specialPad == true
            if specialType == 1

                padW = kernW - 1;
                padH = kernH - 1;
                if rem(padW,2) > 0
                    padW = padW + 1;
                end
                if rem(padH,2) > 0
                    padH = padH + 1;
                end

                padImg = zeros(2*imgHight,2*imgWidth);

                if invertKern == true;
                %%% for (fft(gaussian))^-1 kernel
                    padImg((imgHight+1):(2*imgHight),(imgWidth+1):(2*imgWidth)) = oldImg; % bottom right corner
                    padImg((imgHight+1):(2*imgHight),1:imgWidth) = oldImg(1:imgHight,imgWidth:-1:1); % bottom left corner
                    padImg(1:imgHight,(imgWidth+1):(2*imgWidth)) = oldImg(imgHight:-1:1,1:imgWidth); % top right corner
                    padImg(1:imgHight,1:imgWidth) = oldImg(imgHight:-1:1,imgWidth:-1:1); % top left corner

                    remvPad = @(a) a((imgHight-(padH/2)+1):((2*imgHight)-(padH/2)),(imgWidth-(padW/2)+1):((2*imgWidth)-(padW/2)));

                else 
                %%% for fft(gaussian) kernel
                    padImg((imgHight+1):(2*imgHight),(imgWidth+1):(2*imgWidth)) = oldImg(imgHight:-1:1,imgWidth:-1:1); % bottom right corner
                    padImg((imgHight+1):(2*imgHight),1:imgWidth) = oldImg(imgHight:-1:1,1:imgWidth); % bottom left corner
                    padImg(1:imgHight,(imgWidth+1):(2*imgWidth)) = oldImg(1:imgHight,imgWidth:-1:1); % top right corner
                    padImg(1:imgHight,1:imgWidth) = oldImg; % top left corner

                    remvPad = @(a) a(((padH/2)+1):((padH/2)+imgHight),((padW/2)+1):((padW/2)+imgWidth));

                end

                padIH = size(padImg,1);
                padIW = size(padImg,2);
            end

        elseif reflectPad == true
            if reflectType == 1

                padW = kernW - 1;
                padH = kernH - 1;
                if rem(padW,2) > 0
                    padW = padW + 1;
                end
                if rem(padH,2) > 0
                    padH = padH + 1;
                end

                padImg = zeros((imgHight+(padH)),(imgWidth+(padW)));
                padImg(1:imgHight,1:imgWidth) = oldImg;

                for ia = (imgHight+1):(imgHight+padH)
                    if ia < (imgHight+(padH/2))
                        padImg(ia,:) = padImg((imgHight),:);
                    elseif ia >= (imgHight+(padH/2))
                        padImg(ia,:) = padImg((1),:);
                    end
                end
                for ib = (imgWidth+1):(imgWidth+padW)
                    if ib < (imgWidth+(padW/2))
                        padImg(:,ib) = padImg(:,(imgWidth));
                    elseif ib >= (imgWidth+(padW/2))
                        padImg(:,ib) = padImg(:,(1));
                    end
                end

                padIH = size(padImg,1);
                padIW = size(padImg,2);

                remvPad = @(a) a(((padH/2)+1):((padH/2)+imgHight),((padW/2)+1):((padW/2)+imgWidth));

            elseif reflectType == 2

                padW = kernW - 1;
                padH = kernH - 1;
                if rem(padW,2) > 0
                    padW = padW + 1;
                end
                if rem(padH,2) > 0
                    padH = padH + 1;
                end

                padImg = zeros((imgHight+(padH)),(imgWidth+(padW)));
                padImg(((padH/2)+1):((padH/2)+imgHight),((padW/2)+1):((padW/2)+imgWidth)) = oldImg;

                for ia = 1:(padH/2)
                    padImg(ia,:) = padImg(((padH/2)+1),:);
                end
                for ib = ((padH/2)+imgHight+1):(padH+imgHight)
                    padImg(ib,:) = padImg(((padH/2)+imgHight),:);
                end

                for ic = 1:(padW/2)
                    padImg(:,ic) = padImg(:,((padW/2)+1));
                end
                for id = ((padW/2)+imgWidth+1):(padW + imgWidth)
                    padImg(:,id) = padImg(:,((padW/2)+imgWidth));
                end
                padIH = size(padImg,1);
                padIW = size(padImg,2);

                remvPad = @(a) a((1:imgHight),(1:imgWidth));

            elseif reflectType == 3

                padW = kernW - 1;
                padH = kernH - 1;
                if rem(padW,2) > 0
                    padW = padW + 1;
                end
                if rem(padH,2) > 0
                    padH = padH + 1;
                end

                padImg = zeros((imgHight+(padH)),(imgWidth+(padW)));
                padImg(1:imgHight,1:imgWidth) = oldImg;

                padImg((imgHight+1):end,:) = padImg((imgHight):-1:(imgHight+1-padH),:);
                padImg(:,(imgWidth+1):end) = padImg(:,(imgWidth):-1:(imgWidth+1-padW));

                padIH = size(padImg,1);
                padIW = size(padImg,2);

                remvPad = @(a) a(((padH/2)+1):((padH/2)+imgHight),((padW/2)+1):((padW/2)+imgWidth));

            end

        else % elseif reflectPad == false && specialPad == false

            padW = kernW - 1;
            padH = kernH - 1;
            if rem(padW,2) > 0
                padW = padW + 1;
            end
            if rem(padH,2) > 0
                padH = padH + 1;
            end

            padIW = imgWidth + padW;
            padIH = imgHight + padH;

            padImg = oldImg;
            padImg(padIH,padIW) = 0;

            remvPad = @(a) a(((padH/2)+1):((padH/2)+imgHight),((padW/2)+1):((padW/2)+imgWidth));
        end
    end
    
    if untainted == false
        padKern = kernel;
        padKern(padIH,padIW) = 0;
    elseif keepSize == true
        padKern = kernel;
    else
        if zeroPad == true
            padVal = 0;
        else
            padVal = (mean(kernel(1:kernH,1)) + mean(kernel(1:kernH,kernW)) + mean(kernel(1,1:kernW)) + mean(kernel(kernH,1:kernW)))/4; % gives the average intensity valu of the kernel border.
        end
        padKern = zeros(padIH,padIW);
        padKern(:) = padVal;
        stY = floor(padIH/2)+1-floor(kernH/2);
        stX = floor(padIW/2)+1-floor(kernW/2);
        edY = stY + kernH - 1;
        edX = stX + kernW - 1;
        padKern(stY:edY,stX:edX) = kernel;
    end
    
    FTpadImg = fft2(padImg);
    if untainted == false
        FTpadKern = fft2(padKern);
    else
        FTpadKern = padKern;
    end
    if invertKern == true && untainted == false
        FTpadKern = FTpadKern.^-1;
    end
    FTpadResult = FTpadImg.*FTpadKern;
    padResult = ifft2(FTpadResult);
    imgResult = remvPad(padResult);
    newImgA = real(imgResult);
    
elseif strcmp(method,'simple')
    Hpad = floor(kernH/2);
    Wpad = floor(kernW/2);

    padImg = zeros((imgHight+(2*Hpad)),(imgWidth+(2*Wpad)));
    padImg((1+Hpad):(end-Hpad),(1+Wpad):(end-Wpad)) = oldImg;

    padImgH = size(padImg,1);
    padImgW = size(padImg,2);
    imgX = zeros(imgHight,imgWidth);
    HimgX = size(imgX,1);
    WimgX = size(imgX,2);

    newImgA = imgX;
    for krow = 1:kernH
        for kcol = 1:kernW
            newImgA(:,:) = newImgA(:,:) + kernel(krow,kcol)*padImg((1+krow-1):(HimgX+krow-1),(1+kcol-1):(WimgX+kcol-1));
        end
    end
end


if blockGraph == false
    ax1 = subplot(2,1,1);
    ax2 = subplot(2,1,2);
    colormap(ax1,'gray');
    colormap(ax2,'gray');
    imagesc(imageA,'parent',ax1);
    imagesc(newImgA,'parent',ax2);
end

newImg = newImgA;

% % imagesc(newImgA);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% colMap = cool(100);
% fig1 = figure; 
% % in matlab axes apear to represent the whole plot
% ax1 = axes('parent',fig1); % declare 2 axes (ax1 and ax2) and asign both fig1 as thier parent
% colormap(ax1,'gray'); % asign desired colourmaps to each axes/plot
% ax2 = axes('parent',fig1);
% colormap(ax2,colMap); % I think when only setting the colormap this way, it causes it the colormap to become re-scaled? and leads to an undesirable result.
% set(ax1,'visible','off'); % set the axes of both "axes" (i.e. plots ax1 and ax2) to invisible to prevent ovelaping of interval lables 
% set(ax2,'visible','off');
% HmapIm = imshow(newImgA,colMap,'parent',ax2); % give the resized heatmap the appropriate color map and make ax2 the "parent" of the resized heat map and display the image and assign a handle (HmapIm).
% set(HmapIm,'AlphaData',0.3); % change the alpha value of the displayed heatmap to make it partialy transparent
% background = imageA;
% imshow(background,'parent',ax1); % make ax1 the "parent" of the background image and show and display the background image
% linkaxes([ax1,ax2]); % link the 2 plots tyogether
% figXYWH = get(ax2,'position');
% cbarX = 7*figXYWH(1); % figXYWH(1); % if want on left use "figXYWH(1)", if want on right use "7*figXYWH(1)"
% cbarY = figXYWH(2);
% cbarW = 0.03*figXYWH(3);
% cbarH = figXYWH(4);
% Cbar = colorbar(ax2,'Location','east','position',[cbarX,cbarY,cbarW,cbarH]); % place the color bar as not to obscure the plot
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
 