% masker(imageMatrix,cuttOff)
% =========================================================================
% _____input_____
%
% imageMatrix:
%   -numerical 2D matrix. Is converted to the absolute values within the
%   function.
% cuttOff:
%   -scalar value. Conveted to absolute value within the function.
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% EXAMPLE:
%   -In the case of forming a mask over a electrode wire in a 
%   microscope image. General instructions: First "invert" the greyscale 
%   image before inputting into this function. It is usualy a good idea 
%   to blur the image with 'imgFilterF' after the invert step and prior 
%   to using this function. Work out the most appropriate cutt off by 
%   trial and error, a good starting point is ~200.  
% =========================================================================
% _____output_____
%
% mask: 
%   -logic 0/1 matrix. absolute values of the input matrix
%   ('imageMatrix'), above and equal too the 'cuttOff' absolute value 
%   become 1 and thoes below become 0.
% =========================================================================

function mask = masker(imageMatrix,cuttOff)

imgA = abs(imageMatrix);
threshold = abs(cuttOff);
[rows,cols] = size(imgA);
mask = zeros(rows,cols);
mask(imgA>=threshold) = 1;
