% parameterChecker(targetParameters, imageX, maskX, params)
%
% ==== Input ==== % 
%
% <><><><><><> Required Functions <><><><><> %
% > getGrayscale
% > showLocationsZ
% > excludeMasked
% > findObjectsX
% <><><><><><><><><><><><><><><><><><><><><> %
%
% --- Required --- %
%
% targetParameters: - cell array with 5-7 columns and variable rows.
%                   - each row represents a seperate parameter set used in the
%                     findObjectX function.
%        [required] - Column 1 = filterA deviation (lowerBound length) <2>.
%        [required] - Column 2 = filterB deviation (upperBound length) <3>.
%        [required] - Column 3 = intensity threshold <4.5>.
%        [required] - Column 4 = exclusion length (clossest distance in pixels 
%                     2 points found using the same parameters can be) <20>.
%        [required] - Column 5 = invert (i.e. 255 - image) <false>.
%        [optional] ~ Column 6 = plot symbol <'r'>.
%        [optional] ~ Column 7 = plot color <'o'>.
%
% imageX: - greyscale image with intensity ranges from 0 - 255.
%
% --- optional --- %
%
% maskX: -bool/01 matrix with 1 representing blocked pixels and 0 for
%        accesible pixels
%
% params.index: - if imageX is a folder of images index chooses the image <1>.

function parameterChecker(targetParameters, imageX, maskX, params)

% <><><> Defaults <><><> %

colorLib = {'r','b','g','c','m','y'};
symbolLib = {'o','s','^','d','v','p'};
index = 1;
useMask = false;
maskExist = false;
targetN = 1;
% <><><> Defaults END <><><> %

if nargin < 2
   disp('<parameterChecker> Error: not enough input arguments!');
   return;
end

[targRow,targCol] = size(targetParameters);

if ~iscell(targetParameters) || targCol < 5
    disp('<parameterChecker> Error: "targetParameters" must be a cell array with a minimum of 5 columns!');
   return;
end

targetN = targRow;


if nargin > 2
    if length(maskX(1,:)) > 1
        maskExist = true;
        useMask = maskX;
    end
end

if nargin > 3
    if isfield(params,'index')
        index = params.index;
    end
end

newImg = getGrayscale(imageX, index);

newImgI = 255 - newImg;

rawPositionTables = cell(targetN,1);
optionsX.mask = false;
paramListX = struct;

for i = 1:targetN
    
    if targetParameters{i,5}
        searchImg = newImgI;
    else
        searchImg = newImg;
    end
    
    paramListX.A.deviation = targetParameters{i,1};
    paramListX.B.deviation = targetParameters{i,2};
    paramListX.threshold = targetParameters{i,3};
    paramListX.exclusionLength = targetParameters{i,4};

    rawPositionTables{i} = findObjectsX(searchImg, paramListX, optionsX);
    if maskExist
        rawPositionTables{i} = excludeMasked(rawPositionTables{i},useMask);
    end
    
end

cellData = cell(targetN,3);
cellData(:,1) = rawPositionTables(:,1);

if targCol > 5
    cellData(:,2) = targetParameters(:,6);
    if targCol > 6
        cellData(:,3) = targetParameters(:,7);
    else
        % ~~~ last resort start ~~~ %
        if length(symbolLib(:)) <= targetN
            cellData(:,3) = symbolLib(1:targetN);
        else
            i2 = 0;
            for i = 1:targetN
                if i2 < length(symbolLib(:))
                   i2 = i2 + 1;
                else
                   i2 = 1; 
                end
                cellData(i,3) = symbolLib(i2);
            end
        end
        % ~~~ last resort end ~~~ %
    end
else
    % ~~~ last resort start ~~~ %
    if length(colorLib(:)) <= targetN
        cellData(:,2) = colorLib(1:targetN);
    else
        i2 = 0;
        for i = 1:targetN
            if i2 < length(colorLib(:))
               i2 = i2 + 1;
            else
               i2 = 1; 
            end
            cellData(i,2) = colorLib(i2);
        end
    end
    % ~~~
    if length(symbolLib(:)) <= targetN
        cellData(:,3) = symbolLib(1:targetN);
    else
        i2 = 0;
        for i = 1:targetN
            if i2 < length(symbolLib(:))
               i2 = i2 + 1;
            else
               i2 = 1; 
            end
            cellData(i,3) = symbolLib(i2);
        end
    end
    % ~~~ last resort end ~~~ %
end
     
showLocationsZ(newImg, cellData);
        
end


