% showLocationsZ(image, coordinates, params)

% coordinateData: matrix of XY co-ordinates or cells with each row representing 
%               a seperate plot.
% > using cells: col 1 = each cell contains 2 column matrix of XY
%                        coordinates
%                col 2 = each cell contains char/string representing plot 
%                        color
%                col 3 = each cell contains char/string representing plot
%                        marker shape/symbol
%
% params.rowCol (false): if true order is row, col, if false order is x
%                        (col), y (row)
% params.color ('r'): char/string representing plot color
% params.symbol ('x'): char/string representing plot marker shape
% note - if accepting a table of coordinates from the pkfnd.m funcion, keep
%        this as false (default)
% params.extra (false): if true adds plots to the previos held axis.

function showLocationsZ(image, coordinateData, params)

% ---- Defaults ---- %

rowCol = false;
color = 'r';
symbol = 'x';
usingCell = false;
plotN = 1;


if nargin > 1
    usingCell = iscell(coordinateData); 
   [coRows,coCols] = size(coordinateData);
   
   
   if usingCell
       plotN = coRows;
       coordinateCell = cell(plotN,1);
       colorCell = cell(plotN,1);
       symbolCell = cell(plotN,1);
       coordinateCell(:,1) = coordinateData(:,1);
       if coCols > 1
           colorCell(:,1) = coordinateData(:,2);
           if coCols > 2
               symbolCell(:,1) = coordinateData(:,3);
           else
               symbolCell(:,1) = {'o'};
           end
       else
           colorCell(:,1) = {'r'};
           symbolCell(:,1) = {'o'};
       end
   else
       coordinateCell{1} = coordinateData;
       colorCell = cell(1);
       symbolCell = cell(1);
       colorCell(:,1) = {'r'};
       symbolCell(:,1) = {'o'};
   end
end

if nargin > 2
   if isfield(params,'rowCol')
      rowCol = params.rowCol;
   end
   if isfield(params,'color')
       if iscell(params.color)
           colorCell(:,1) = params.color(:);
       else
           colorCell(:,1) = {params.color};
       end 
   end
   if isfield(params,'symbol')
       if iscell(params.symbol)
           symbolCell(:,1) = params.symbol(:);
       else
           symbolCell(:,1) = {params.symbol};
       end 
   end
end

colorSymb = [color,symbol];

if rowCol
   colX = 2;
   colY = 1;
else
   colX = 1;
   colY = 2;
end

figA = figure;
axA = axes('parent',figA);
colormap(axA, 'gray');
imagesc(image,'parent',axA);
hold on
for i = 1:plotN
    plot(axA, coordinateCell{i}(:,colX),coordinateCell{i}(:,colY),[colorCell{i},symbolCell{i}]);
end


end